using System;
        using System.Collections.Generic;
        using System.Net.Http;
        using System.Text.Json;
        using System.Threading;
        using System.Threading.Tasks;
        using Godot;
        using HttpClient = System.Net.Http.HttpClient;
        
        namespace SessionZeroClient.API;
        
        public class ApiHandler : IApiHandler
        {
            private readonly HttpClient _httpClient;
        
            public ApiHandler()
            {
                var handler = new HttpClientHandler
                {
                    UseDefaultCredentials = true,
                };
        
                _httpClient = new HttpClient(handler)
                {
                    Timeout = TimeSpan.FromSeconds(5),
                };
            }
        
            public async Task<(OperationStatus, bool)> ValidateServerUrl(string url)
            {
                try
                {
                    var apiString = $"{url}/api/Database/validate-server";
        
                    using var cts = new CancellationTokenSource(TimeSpan.FromSeconds(5));
                    var response = await _httpClient.GetAsync(apiString, cts.Token);
        
                    if (!response.IsSuccessStatusCode)
                    {
                        return (OperationStatus.ServerNotReachable, false);
                    }
        
                    var responseString = await response.Content.ReadAsStringAsync(cts.Token);
        
                    var options = new JsonSerializerOptions { PropertyNameCaseInsensitive = true };
                    var validationResponse = JsonSerializer.Deserialize<ServerUrlValidationResponse>(responseString, options);
        
                    return (OperationStatus.Success, validationResponse?.Message ?? false);
                }
                catch
                {
                    return (OperationStatus.ServerNotReachable, false);
                }
            }
        
            public async Task<(OperationStatus, string)> Register(string url, string username, string email, string password)
            {
                var (status, isValidServer) = await ValidateServerUrl(url);
                if (!isValidServer)
                {
                    return (status, null);
                }
        
                var apiString = $"{url}/api/Auth/register";
                var requestBody = new Dictionary<string, string>
                {
                    {"username", username},
                    {"email", email},
                    {"password", password}
                };
        
                var jsonBody = JsonSerializer.Serialize(requestBody);
                var content = new StringContent(jsonBody, System.Text.Encoding.UTF8, "application/json");
                var response = await _httpClient.PostAsync(apiString, content);
        
                var responseString = await response.Content.ReadAsStringAsync();
        
                try
                {
                    var responseData = JsonSerializer.Deserialize<Dictionary<string, string>>(responseString);
                    if (responseData != null && responseData.ContainsKey("message") && responseData["message"] == "User registered successfully")
                    {
                        return (OperationStatus.Success, responseString);
                    }
                }
                catch (JsonException)
                {
                    try
                    {
                        var errorResponse = JsonSerializer.Deserialize<List<ValidationError>>(responseString);
                        if (errorResponse != null)
                        {
                            return (OperationStatus.ValidationError, JsonSerializer.Serialize(errorResponse));
                        }
                    }
                    catch (JsonException)
                    {
                        var errorResponse = JsonSerializer.Deserialize<ErrorResponse>(responseString);
                        if (errorResponse != null)
                        {
                            return (OperationStatus.GeneralError, JsonSerializer.Serialize(errorResponse));
                        }
                    }
                }
        
                return (OperationStatus.GeneralError, null);
            }
        
            public async Task<(OperationStatus, string)> Login(string url, string username, string password)
            {
                var (status, isValidServer) = await ValidateServerUrl(url);
                if (!isValidServer)
                {
                    return (status, null);
                }
        
                var apiString = $"{url}/api/Auth/login";
                var requestBody = new Dictionary<string, string>
                {
                    {"username", username},
                    {"password", password}
                };
        
                var jsonBody = JsonSerializer.Serialize(requestBody);
                var content = new StringContent(jsonBody, System.Text.Encoding.UTF8, "application/json");
                var response = await _httpClient.PostAsync(apiString, content);
        
                var responseString = await response.Content.ReadAsStringAsync();
        
                try
                {
                    var responseData = JsonSerializer.Deserialize<Dictionary<string, string>>(responseString);
                    if (responseData != null && responseData.ContainsKey("token"))
                    {
                        return (OperationStatus.Success, responseString);
                    }
                }
                catch (JsonException)
                {
                    var errorResponse = JsonSerializer.Deserialize<ErrorResponse>(responseString);
                    if (errorResponse != null && errorResponse.Status == 401)
                    {
                        return (OperationStatus.Unauthorized, JsonSerializer.Serialize(errorResponse));
                    }
                }
        
                return (OperationStatus.GeneralError, null);
            }
        
            public class ServerUrlValidationResponse
            {
                public bool Message { get; set; }
            }
        
            public class ErrorResponse
            {
                public string Type { get; set; }
                public string Title { get; set; }
                public int Status { get; set; }
                public string TraceId { get; set; }
            }
        
            public class ValidationError
            {
                public string Code { get; set; }
                public string Description { get; set; }
            }
        }
        
        public enum OperationStatus
        {
            Success,
            ServerNotReachable,
            Unauthorized,
            ValidationError,
            GeneralError
        }