using System;
using Godot;
using SessionZeroClient.API;
using SessionZeroClient.Controls;
using SessionZeroClient.Resources;
using SessionZeroClient.Screens;

namespace SessionZeroClient;

public class AppStateManager
{
    public AppStateManagerSettings Settings;
    public Main MainNode;
    public IApiHandler ApiHandler;
    public AccountManager AccountManager;
    
    // Screen Logics
    public AccountScreenLogic AccountScreenLogic { get; private set; }
    public MainScreenLogic MainScreenLogic { get; private set; }
    public HomeScreenLogic HomeScreenLogic { get; private set; }
    
    // Control Logic
    public NavigationControlLogic NavigationControlLogic { get; private set; }
    
    public AppStateManager(AppStateManagerSettings settings, Main mainNode)
    {
        Settings = settings;
        MainNode = mainNode;
        
        ApiHandler = new ApiHandler();
        AccountManager = new AccountManager(this);
        
        CreateAccountScreen();
        CreateMainScreen();
        CreateHomeScreen();
        CreateNavMenu();
    }
    
    // Account Screen
    private void CreateAccountScreen()
    {
        if (Settings.AccountScreenScene == null)
        {
            throw new Exception("Account screen scene export is null");
        }

        try
        {
            AccountScreenLogic = Settings.AccountScreenScene.Instantiate<AccountScreenLogic>();
        }
        catch (Exception e)
        {
            Console.WriteLine("Problem with creating the Account Screen Logic: ", e);
            throw;
        }
        
        AccountScreenLogic.Init(this);
    }

    public void ShowAccountScreen()
    {
        if (AccountScreenLogic == null) CreateAccountScreen();
        
        ChangeState(AccountScreenLogic);
    }
    
    // Main Screen
    private void CreateMainScreen()
    {
        if (Settings.MainScreenScene == null)
        {
            throw new Exception("Main screen scene export is null");
        }

        try
        {
            MainScreenLogic = Settings.MainScreenScene.Instantiate<MainScreenLogic>();
        }
        catch (Exception e)
        {
            Console.WriteLine("Problem with creating the Main Screen Logic: ", e);
            throw;
        }
    }

    public void ShowMainScreen()
    {
        if (MainScreenLogic == null) CreateMainScreen();
        
        ChangeState(MainScreenLogic);
    }

    // Home Screen

    private void CreateHomeScreen()
    {
        if (Settings.HomeScreenScene == null)
        {
            Console.WriteLine("Home screen scene export is null");
            return;
        }

        try
        {
            HomeScreenLogic = Settings.HomeScreenScene.Instantiate<HomeScreenLogic>();
        }
        catch (Exception e)
        {
            Console.WriteLine("Problem creating Home Screen Logic: ", e);
            throw;
        }
        
        HomeScreenLogic.Init(this);
    }
    
    public void ShowHomeScreen()
    {
        if (HomeScreenLogic == null) CreateHomeScreen();
        
        MainScreenLogic.ChangeCenterContent(HomeScreenLogic);
    }

    public void RefreshHomeScreen()
    {
        HomeScreenLogic.Refresh();
    }
    
    // Nav menu
    public void CreateNavMenu()
    {
        if (Settings.NavigationControl == null)
        {
            Console.WriteLine("Navigation control export is null");
            return;
        }

        try
        {
            NavigationControlLogic = Settings.NavigationControl.Instantiate<NavigationControlLogic>();
        }
        catch (Exception e)
        {
            Console.WriteLine("Problem with creating the Navigation Control Logic: ", e);
            throw;
        }
        
        NavigationControlLogic.Init(this);
    }
    
    // States
    public void ChangeState(Control screen)
    {
        if (MainNode.MainCanvasLayer.GetChildren().Count > 0)
        {
            MainNode.MainCanvasLayer.RemoveChild(MainNode.MainCanvasLayer.GetChild(0));
        }
        
        MainNode.MainCanvasLayer.AddChild(screen);
    }
}

